<?php
/**
 * Provides logging capabilities for debugging purposes.
 *
 * @package    WPTelegram\Pro
 * @subpackage WPTelegram\Pro\includes
 */

namespace WPTelegram\Pro\includes;

use ReflectionClass;
use WP_Filesystem_Base;
use WPTelegram\BotAPI\Response;
use WPTelegram\BotAPI\API;
use WP_Post;
use WPTelegram\Pro\modules\p2tg\RequestCheck;
use WPTelegram\Pro\modules\p2tg\Main as P2TGMain;

/**
 * Provides logging capabilities for debugging purposes.
 */
class Logger extends BaseClass {

	/**
	 * Enabled Log types
	 *
	 * @since   1.0.0
	 * @access  private
	 * @var     array       $active_logs    The enabled logs
	 */
	private static $active_logs;

	/**
	 * Whether already hooked or not.
	 *
	 * @since   1.0.0
	 * @access  private
	 * @var     array       $hooked_up  The enabled logs
	 */
	private static $hooked_up = false;

	/**
	 * Information about the processed post
	 *
	 * @since   1.0.0
	 * @access  private
	 * @var     array       $p2tg_post_info     The Post info
	 */
	private $p2tg_post_info;

	/**
	 * Set the active logs.
	 *
	 * @param array $active_logs The logs that are activated.
	 *
	 * @return self
	 */
	public function set_active_logs( $active_logs ) {

		self::$active_logs = (array) $active_logs;

		return $this;
	}

	/**
	 * Get the active logs
	 */
	public function get_active_logs() {

		return (array) apply_filters( 'wptelegram_pro_logger_active_logs', self::$active_logs );
	}

	/**
	 * Hook into WP Telegram to create logs.
	 */
	public function hookup() {

		add_action( 'init', [ $this, 'view_log' ] );

		// avoid hooking in multiple times.
		if ( ! self::$hooked_up && ! empty( self::$active_logs ) ) {

			$this->hook_it_up();

			self::$hooked_up = true;
		}
	}

	/**
	 * Hook into WP Telegram to create logs.
	 */
	protected function hook_it_up() {

		$active_logs = $this->get_active_logs();

		if ( empty( $active_logs ) ) {
			return;
		}

		foreach ( $active_logs as $log_type ) {

			$method = [ $this, "hookup_for_{$log_type}" ];

			if ( is_callable( $method ) ) {
				call_user_func( $method );
			}
		}

		add_action( 'wptelegram_pro_prepare_content_error', [ $this, 'prepare_content_error' ], 10, 3 );
	}

	/**
	 * Get the URL for a log type.
	 *
	 * @param string $type Log type.
	 * @return string
	 */
	public static function get_log_url( $type ) {

		$url = add_query_arg(
			[
				'action' => 'wptelegram_pro_view_log',
				'hash'   => wp_hash( 'log' ),
				'type'   => $type,
			],
			site_url()
		);

		return apply_filters( 'wptelegram_pro_logger_log_url', $url, $type );
	}

	/**
	 * View logs
	 */
	public function view_log() {

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( isset( $_GET['action'], $_GET['hash'], $_GET['type'] ) && 'wptelegram_pro_view_log' === $_GET['action'] && isset( $_GET['hash'] ) ) {
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$hash = sanitize_text_field( wp_unslash( $_GET['hash'] ) );
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$type = sanitize_text_field( wp_unslash( $_GET['type'] ) );

			if ( ! empty( $hash ) && ! empty( $type ) ) {

				$file_path = self::get_log_file_path( $type, $hash );

				if ( self::wp_filesystem()->exists( $file_path ) ) {
					$contents = self::wp_filesystem()->get_contents( $file_path );
				} else {
					$contents = 'Log file not found!';
				}

				header( 'Content-Type: text/plain' );

				// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				exit( $contents );
			}
		}
	}

	/**
	 * Hook for \WPTelegram\BotAPI log.
	 */
	protected function hookup_for_bot_api_out() {

		add_action( 'wptelegram_bot_api_debug', [ $this, 'add_bot_api_debug' ], 10, 2 );
	}

	/**
	 * Hook for incoming bot updates log.
	 */
	protected function hookup_for_bot_api_in() {

		add_action( 'wptelegram_pro_bots_webhook_update_init', [ $this, 'add_webhook_update_init' ], 10, 1 );
		add_action( 'wptelegram_pro_bots_after_get_update', [ $this, 'add_bots_after_update' ], 10, 4 );
	}

	/**
	 * Hook for Post to Telegram log
	 */
	protected function hookup_for_p2tg() {

		add_action( 'wptelegram_pro_p2tg_before_send_post', [ $this, 'add_before_send' ], 999, 5 );

		add_action( 'wptelegram_pro_p2tg_set_form_data', [ $this, 'set_form_data' ], 999, 2 );

		add_action( 'wptelegram_pro_p2tg_post_sv_check_failed', [ $this, 'add_sv_check' ], 999, 2 );

		add_action( 'wptelegram_pro_p2tg_before_process', [ $this, 'add_before_process' ], 999, 2 );

		add_action( 'wptelegram_pro_p2tg_set_instance_for_delay', [ $this, 'add_set_instance_for_delay' ], 999, 4 );

		add_filter( 'wptelegram_pro_p2tg_responses', [ $this, 'add_get_responses' ], 999, 3 );

		add_filter( 'wptelegram_pro_p2tg_post_date_rules_apply', [ $this, 'add_date_rules_apply' ], 999, 3 );
		add_filter( 'wptelegram_pro_p2tg_bypass_post_date_rules', [ $this, 'add_bypass_post_date_rules' ], 999, 3 );
		add_filter( 'wptelegram_pro_p2tg_rules_send_new_post', [ $this, 'add_send_new_post' ], 999, 3 );
		add_filter( 'wptelegram_pro_p2tg_rules_send_existing_post', [ $this, 'add_send_existing_post' ], 999, 3 );
		add_filter( 'wptelegram_pro_p2tg_rules_send_post_type', [ $this, 'add_send_post_type' ], 999, 3 );
		add_filter( 'wptelegram_pro_p2tg_post_type_rules_apply', [ $this, 'add_post_type_rules_apply' ], 999, 3 );
		add_filter( 'wptelegram_pro_p2tg_dynamic_rules_apply', [ $this, 'add_dynamic_rules_apply' ], 999, 3 );
		if ( defined( 'WPTELEGRAM_PRO_DEBUG_DYNAMIC_RULES' ) && constant( 'WPTELEGRAM_PRO_DEBUG_DYNAMIC_RULES' ) ) {
			add_filter( 'wptelegram_pro_p2tg_dynamic_rule_matches', [ $this, 'add_dynamic_rule_matches' ], 999, 5 );
		}

		add_filter( 'wptelegram_pro_p2tg_featured_image_source', [ $this, 'add_featured_image_source' ], 999, 4 );

		add_action( 'wptelegram_pro_p2tg_after_process', [ $this, 'add_after_process' ], 10, 3 );

		add_action( 'wptelegram_pro_p2tg_send_post_finish', [ $this, 'add_post_finish' ], 999, 5 );

		add_action( 'wptelegram_pro_p2tg_after_send_post', [ $this, 'add_after_send' ], 999, 3 );
	}

	/**
	 * Get the current request type.
	 *
	 * @param WP_Post $post The post being handled.
	 */
	private function get_request_type( $post ) {

		$request_check = new ReflectionClass( RequestCheck::class );

		$constants = $request_check->getConstants();

		foreach ( $constants as $constant => $value ) {
			if ( RequestCheck::is( $value, $post ) ) {
				return $constant;
			}
		}
	}

	/**
	 * Get the key from post.
	 *
	 * @param WP_Post $post The post being handled.
	 */
	public function get_key( $post ) {
		return $post->post_type . '-' . $post->ID . '-' . $post->post_status;
	}

	/**
	 * Handle the debug action.
	 *
	 * @param Response $response  The API response.
	 * @param API      $tg_api    The post being handled.
	 */
	public function add_bot_api_debug( $response, $tg_api ) {

		$res = $tg_api->get_last_response();

		$params = $tg_api->get_request()->get_params();

		// add the method and request params.
		$text = 'Method: ' . $tg_api->get_request()->get_api_method() . PHP_EOL . 'Params: ' . $this->json_encode( $params, 'bot-api' ) . PHP_EOL . '--------------------------------' . PHP_EOL;

		// add the response.
		if ( is_wp_error( $res ) ) {
			$text .= 'WP_Error: ' . $res->get_error_code() . ' ' . $res->get_error_message() . PHP_EOL;

			$base_url = $tg_api->get_client()->get_base_url();
			// redact the worker name if present.
			$base_url = preg_replace( '/(?<=https:\/\/)[^\.]+?(?=\.)/', '***', $base_url );

			$text .= 'URL: ' . $base_url;
		} else {
			$text .= 'Response: ' . $this->json_encode( json_decode( $res->get_body(), true ), 'bot-api' );
		}

		$this->write_log( 'bot-api', $text );
	}

	/**
	 * Add the incoming bot updates init hook.
	 *
	 * @param string $bot_token Bot token.
	 *
	 * @return void
	 */
	public function add_webhook_update_init( $bot_token ) {

		$text = 'webhook-init: ' . substr( $bot_token, 0, 10 );

		$this->write_log( 'bot-updates', $text );
	}

	/**
	 * Add the incoming bot updates after hook.
	 *
	 * @param array  $update        The update object.
	 * @param string $bot_token     Bot token.
	 * @param string $update_method The update method.
	 * @param array  $bots          Bot collection.
	 *
	 * @return void
	 */
	public function add_bots_after_update( $update, $bot_token, $update_method, $bots ) {

		if ( ! empty( $update ) ) {
			$text = $bots[ $bot_token ] . ' ' . $this->json_encode( $update, 'bot-updates' );

			$this->write_log( 'bot-updates', $text );
		}
	}

	/**
	 * Handle p2tg before post send action.
	 *
	 * @param mixed   $result   The action result.
	 * @param WP_Post $post     The post being handled.
	 * @param string  $trigger  The source trigger.
	 * @param array   $inc_inst Included instances.
	 * @param bool    $force    Whether to force send.
	 */
	public function add_before_send( $result, $post, $trigger, $inc_inst, $force ) {
		// `is_new_post` is in a static class, so we want to add it only when Sending to Telegram.
		add_filter( 'wptelegram_pro_p2tg_is_post_new', [ $this, 'add_is_new_post' ], 999, 4 );

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['before_send'] = [
			'trigger'      => $trigger,
			'inc_inst'     => $inc_inst,
			'force'        => $force,
			'request_type' => $this->get_request_type( $post ),
		];
	}

	/**
	 * Add form data to the log.
	 *
	 * @param mixed   $form_data The action result.
	 * @param WP_Post $post      The post being handled.
	 */
	public function set_form_data( $form_data, $post ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		// Remove message_template to make the logs clean.
		if ( ! empty( $form_data['instances'] ) && is_array( $form_data['instances'] ) ) {
			$form_data['instances'] = array_map(
				function ( $instance ) {
					unset( $instance['message_template'] );
					return $instance;
				},
				$form_data['instances']
			);
		}

		$this->p2tg_post_info[ $key ]['form_data'] = $form_data;
	}

	/**
	 * Add security and validity info.
	 *
	 * @param int     $validity The request validity.
	 * @param WP_Post $post     The post being handled.
	 */
	public function add_sv_check( $validity, $post ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['sv'] = $validity;
	}

	/**
	 * Add before process info.
	 *
	 * @param WP_Post $post      The post being handled.
	 * @param array   $instances The instances collection.
	 */
	public function add_before_process( $post, $instances ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['before_process'] = [
			'instances' => array_keys( $instances ),
		];
	}

	/**
	 * Add instance delay info.
	 *
	 * @param float   $delay       Delay in posting.
	 * @param int     $instance_id Instance ID.
	 * @param WP_Post $post        The post being handled.
	 * @param array   $result      The result of delay handler.
	 */
	public function add_set_instance_for_delay( $delay, $instance_id, $post, $result ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['delay'][ $instance_id ] = [
			'duration' => $delay,
			'result'   => $result,
		];
	}

	/**
	 * Add get responses info.
	 *
	 * @param array   $responses The responses collection.
	 * @param WP_Post $post      The post being handled.
	 * @param array   $instances The instances collection.
	 */
	public function add_get_responses( $responses, $post, $instances ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['responses'] = [
			'responses' => $this->shorten_responses( $responses ),
			'instances' => array_keys( $instances ),
		];

		return $responses;
	}

	/**
	 * Add after process info.
	 *
	 * @param WP_Post $post      The post being handled.
	 * @param array   $instances The instances collection.
	 * @param array   $responses The responses collection.
	 */
	public function add_after_process( $post, $instances, $responses ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['after_process'] = [
			'instances' => array_keys( $instances ),
			'responses' => $this->shorten_responses( $responses ),
		];
	}

	/**
	 * Shorten responses.
	 *
	 * @param array $responses The responses.
	 * @return array
	 */
	public function shorten_responses( $responses ) {
		if ( empty( $responses ) ) {
			return $responses;
		}

		foreach ( $responses as $instance_id => $inst_responses ) {
			$responses[ $instance_id ] = array_map(
				function ( $response ) {
					reset( $response );
					return key( $response );
				},
				$inst_responses
			);
		}

		return $responses;
	}

	/**
	 * Add date_rules_apply info
	 *
	 * @param boolean $rules_apply Whether the rules apply.
	 * @param Options $options     The instance options.
	 * @param WP_Post $post        The post being handled.
	 */
	public function add_date_rules_apply( $rules_apply, $options, $post ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $options->get( 'id' ) ]['apply']['date'] = $rules_apply;

		return $rules_apply;
	}

	/**
	 * Add bypass_post_date_rules info.
	 *
	 * @param boolean $bypass_date_rules Whether to bypass date rules.
	 * @param Options $options           The instance options.
	 * @param WP_Post $post              The post being handled.
	 */
	public function add_bypass_post_date_rules( $bypass_date_rules, $options, $post ) {

		// create a an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $options->get( 'id' ) ]['bypass']['date'] = $bypass_date_rules;

		return $bypass_date_rules;
	}

	/**
	 * Add is_new_post info.
	 *
	 * @param boolean $is_new                 Whether the post is new.
	 * @param WP_Post $post                   The post being handled.
	 * @param int     $instance_id            The instance ID.
	 * @param boolean $is_more_than_a_day_old Whether the post is more than a day old.
	 */
	public function add_is_new_post( $is_new, $post, $instance_id, $is_more_than_a_day_old ) {

		// create a an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $instance_id ]['is'] = [
			'new'       => $is_new,
			'a_day_old' => $is_more_than_a_day_old,
			'sent2tg'   => get_post_meta( $post->ID, P2TGMain::PREFIX . 'sent2tg', true ),
		];

		return $is_new;
	}

	/**
	 * Add send_new_post info.
	 *
	 * @param boolean $send_new Whether to send new post.
	 * @param Options $options  The instance options.
	 * @param WP_Post $post     The post being handled.
	 */
	public function add_send_new_post( $send_new, $options, $post ) {

		// create a an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $options->get( 'id' ) ]['send']['new'] = $send_new;

		return $send_new;
	}

	/**
	 * Add send_existing_post info.
	 *
	 * @param boolean $send_existing Whether to send new post.
	 * @param Options $options       The instance options.
	 * @param WP_Post $post          The post being handled.
	 */
	public function add_send_existing_post( $send_existing, $options, $post ) {

		// create a an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $options->get( 'id' ) ]['send']['existing'] = $send_existing;

		return $send_existing;
	}

	/**
	 * Add send_post_type info.
	 *
	 * @param boolean $send_post_type Whether the post is new.
	 * @param Options $options        The instance options.
	 * @param WP_Post $post           The post being handled.
	 */
	public function add_send_post_type( $send_post_type, $options, $post ) {

		// create a an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $options->get( 'id' ) ]['send']['post_type'] = $send_post_type;

		return $send_post_type;
	}

	/**
	 * Add post_type_rules_apply info
	 *
	 * @param boolean $rules_apply Whether the rules apply.
	 * @param Options $options     The instance options.
	 * @param WP_Post $post        The post being handled.
	 */
	public function add_post_type_rules_apply( $rules_apply, $options, $post ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $options->get( 'id' ) ]['apply']['post_type'] = $rules_apply;

		return $rules_apply;
	}

	/**
	 * Add dynamic_rules_apply info
	 *
	 * @param boolean $rules_apply Whether the rules apply.
	 * @param Options $options     The instance options.
	 * @param WP_Post $post        The post being handled.
	 */
	public function add_dynamic_rules_apply( $rules_apply, $options, $post ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $options->get( 'id' ) ]['apply']['dynamic'] = $rules_apply;

		return $rules_apply;
	}

	/**
	 * Add dynamic_rule_matches info
	 *
	 * @param boolean $rule_matches Whether the rule matches.
	 * @param Options $rule         The instance rule.
	 * @param Options $options      The instance options.
	 * @param WP_Post $post         The post being handled.
	 * @param array   $data         The rule data.
	 */
	public function add_dynamic_rule_matches( $rule_matches, $rule, $options, $post, $data ) {

		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['rules'][ $options->get( 'id' ) ]['rule_matches'] = compact( 'rule', 'rule_matches', 'data' );

		return $rule_matches;
	}

	/**
	 * Add rules_apply info
	 *
	 * @param string  $source            The featured image source.
	 * @param WP_Post $post              The post being handled.
	 * @param Options $options           The post being handled.
	 * @param boolean $send_files_by_url The featured image source.
	 */
	public function add_featured_image_source( $source, $post, $options, $send_files_by_url ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['image_source'][ $options->get( 'id' ) ] = [
			'send_image'  => $options->get( 'send_featured_image' ),
			'has_image'   => has_post_thumbnail( $post->ID ),
			'send_by_url' => $send_files_by_url,
			'source'      => $source,
		];

		return $source;
	}

	/**
	 * Add post send finish info.
	 *
	 * @param WP_Post $post            The post being handled.
	 * @param string  $trigger         The source trigger.
	 * @param boolean $ok              The featured image source.
	 * @param array   $inc_inst        Included instances.
	 * @param array   $processed_posts The featured image source.
	 */
	public function add_post_finish( $post, $trigger, $ok, $inc_inst, $processed_posts ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		$this->p2tg_post_info[ $key ]['finish'] = [
			'ok'        => $ok,
			'inc_inst'  => $inc_inst,
			'processed' => $processed_posts,
		];
	}

	/**
	 * Add after send post info.
	 *
	 * @param mixed   $result  The action result.
	 * @param WP_Post $post    The post being handled.
	 * @param string  $trigger The source trigger.
	 */
	public function add_after_send( $result, $post, $trigger ) {

		// create an entry from post ID and its status.
		$key = $this->get_key( $post );

		if ( is_array( $result ) ) {
			$result = array_map(
				function ( $instances ) {
					if ( is_array( $instances ) ) {
						return array_map(
							function ( $responses ) {
								if ( is_array( $responses ) ) {
									return array_map(
										function ( $response ) {
											if ( $response instanceof Response ) {
												return $response->get_decoded_body();
											}
											return $response;
										},
										$responses
									);
								}
								return $responses;
							},
							$instances
						);
					}
					return $instances;
				},
				$result
			);
		}

		$this->p2tg_post_info[ $key ]['after_send'] = [
			'result' => $result,
		];

		$text = $this->json_encode( [ $key => $this->p2tg_post_info[ $key ] ], 'p2tg' );

		$this->write_log( 'p2tg', $text );

		unset( $this->p2tg_post_info[ $key ] );
	}

	/**
	 * Convert the data to JSON with filter.
	 *
	 * @param array  $data The data to encode.
	 * @param string $type The log type.
	 *
	 * @return string
	 */
	private function json_encode( $data, $type = '' ) {
		$params = [ $data ];
		if ( apply_filters( 'wptelegram_pro_logger_pretty_json', false, $type ) ) {
			$params[] = JSON_PRETTY_PRINT;
		}

		return call_user_func_array( 'json_encode', $params );
	}

	/**
	 * Handle prepare content error.
	 *
	 * @param ConverterException $exception The exception thrown.
	 * @param string             $content The content that was being prepared.
	 * @param array              $options The options passed to prepare_content.
	 *
	 * @return void
	 */
	public function prepare_content_error( $exception, $content, $options ) {
		$text  = 'Error: ' . $exception . PHP_EOL;
		$text .= 'Options: ' . wp_json_encode( $options ) . PHP_EOL;
		$text .= 'Content: ' . $content;

		$this->write_log( 'converter', $text );
	}

	/**
	 * Write the log to file.
	 *
	 * @param string $type The log type.
	 * @param string $text The content.
	 */
	public function write_log( $type, $text ) {

		$bot_token_regex = '/' . \WPTelegram\BotAPI\API::BOT_TOKEN_PATTERN . '/';

		$text = preg_replace( $bot_token_regex, '**********', $text );

		$file_path = self::get_log_file_path( $type );

		$filesystem = self::wp_filesystem();

		// Default to 1 MB.
		$max_filesize = apply_filters( 'wptelegram_pro_logger_max_filesize', 1024 ** 2, $type, $file_path );

		$contents  = $filesystem->is_readable( $file_path ) ? $filesystem->get_contents( $file_path ) : '';
		$contents .= '[' . current_time( 'mysql' ) . ']' . PHP_EOL . $text . PHP_EOL . PHP_EOL;

		// Make sure that the file size remains less than $max_filesize.
		while ( mb_strlen( $contents ) > $max_filesize ) {
			$content_pieces = preg_split( "/[\n\r]{2}/", $contents );

			// Remove the first piece.
			array_shift( $content_pieces );

			$contents = implode( "\n\n", $content_pieces );
		}

		$filesystem->put_contents( $file_path, $contents );
	}

	/**
	 * Get an instance of WP_Filesystem.
	 *
	 * @return WP_Filesystem_Base The global WP_Filesystem object.
	 */
	public static function wp_filesystem() {

		require_once ABSPATH . 'wp-admin/includes/file.php';
		WP_Filesystem();

		global $wp_filesystem;

		return $wp_filesystem;
	}

	/**
	 * Get log file path.
	 *
	 * @since 1.4.0
	 *
	 * @param string $type Log type.
	 * @param string $hash The hash to use in file name.
	 *
	 * @return string
	 */
	public static function get_log_file_path( $type, $hash = '' ) {

		$file_name = self::get_log_file_name( $type, $hash );

		$file_path = self::wp_filesystem()->wp_content_dir() . $file_name;

		return apply_filters( 'wptelegram_pro_logger_log_file_path', $file_path, $type );
	}

	/**
	 * Get log file name.
	 *
	 * @since 1.4.0
	 *
	 * @param string $type Log type.
	 * @param string $hash The hash to use in file name.
	 *
	 * @return string
	 */
	public static function get_log_file_name( $type, $hash = '' ) {

		$hash = $hash ? $hash : wp_hash( 'log' );

		$file_name = "wptelegram-pro-{$type}-{$hash}.log";

		return apply_filters( 'wptelegram_pro_logger_log_file_name', $file_name, $type, $hash );
	}
}
